﻿// *                                                               *
// *       Copyright (c) Fast Corporation, 2007                     *
// *                                                               *
// *   All Rights Reserved.  Unpublished rights  reserved  under   *
// *   the copyright laws of the Japan.                            *
// *                                                               *
// *   The software contained on this media  is  proprietary  to   *
// *   and  embodies  the  confidential  technology  of  Fast      *
// *   Corporation.  Possession, use,  duplication  or             *
// *   dissemination of the software and media is authorized only  *
// *   pursuant to a valid written license from Fast Corporation.  *
// *                                                               *
// *****************************************************************

// -------------------------------------------------------------------
// FVX Sample Program
// 
//
// 作成者 : R.Nagashima
// 履歴   : 2007/01/11 初版
//
//-------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.IO;
using FVDATASLib;

namespace Distance
{
    public partial class frmMain : Form
    {
        public frmMain()
        {
            InitializeComponent();
        }

        //処理範囲
        public FvRectangle m_ExeRegion = new FvRectangle();
        // ビデオデバイスがオープンされているか
        private bool m_blnVideoOpened = false;
        // ビデオ設定ファイル名（フルパス）
        private string m_strVideoConfFilePath = "";
        // 初期ディレクトリ
        private string m_strDirectory;

        private void frmMain_Load(object sender, EventArgs e)
        {
            int IntErr;
            frmSelectvideoConfFile fVideoConf = new frmSelectvideoConfFile();
            frmLicense fLicense = new frmLicense();

            // 変数初期化
            m_blnVideoOpened = false;
            btnGrab.Enabled = false;

            cmbColor.Items.Insert(0, "BLACK");
            cmbColor.Items.Insert(1, "WHITE");
            cmbColor.SelectedIndex = 0;

            //画面の生成
            axFvxDisplayEx1.CreateDisplay();

            //処理範囲の初期値設定(画像の全面に設定）
            m_ExeRegion.xs = -1;
            m_ExeRegion.xe = -1;
            m_ExeRegion.ys = -1;
            m_ExeRegion.ye = -1;

            //作業ディレクトリ退避
            m_strDirectory = Directory.GetCurrentDirectory();

            //使用許諾の確認
            if (fLicense.ShowDialog() == DialogResult.OK)
            {
                //ビデオ設定ファイルの選択
                if (fVideoConf.ShowDialog() == DialogResult.OK)
                    m_strVideoConfFilePath = fVideoConf.strVideoConfFilePath;

                if (fVideoConf.chkNotUseCamera.Checked == false)
                {
                    //ビデオ設定ファイルをコンポーネントに設定
                    axFvxVideo1.VideoConfFile = m_strVideoConfFilePath;

                    //デバイスオープン
                    try
                    {
                        axFvxVideo1.OpenDevice();
                    }
                    catch (Exception)
                    {
                        axFvxVideo1.GetErrReport(out IntErr);
                        MessageBox.Show("ビデオデバイスのオープンに失敗しました｡ ErrCode= " + IntErr.ToString() + "\r\n画像ファイルのみの利用になります。");
                        return;
                    }

                    //VideoConfFileの変更後、OpenDeviceを行うと、トリガーモードの変更が伴います
                    //このため、弊社キャプチャボードFVC02では350msのセットアップタイムが必要になります｡
                    //本プログラムでは簡略化のために、一律に350msのセットアップタイムを確保していますが、
                    //お使いのキャプチャボードに合わせて変更してください。
                    Thread.Sleep(350);

                    //GrabSync/GrabASyncメソッドではImegeサイズの変更を行いませんので、
                    //あらかじめChangeImageSizeメソッドにてImageのサイズを変更してください
                    //（取り込みサイズとImageのサイズが違う場合エラーになります）
                    axFvxVideo1.ChangeImageSize(axFvxImageGray.GetOcx());
                    try
                    {
                        // 同期取込
                        axFvxVideo1.GrabSync(axFvxImageGray.GetOcx());
                    }
                    catch (Exception)
                    {
                        axFvxVideo1.GetErrReport(out IntErr);
                        MessageBox.Show("ビデオの取込に失敗しました｡ ErrCode= " + IntErr.ToString() + "\r\n画像ファイルのみの利用になります。");
                        return;
                    }

                    //ビデオデバイスの状態を保存
                    m_blnVideoOpened = true;
                    btnGrab.Enabled = true;

                    //画像を表示
                    axFvxChangeImage1.FixedLevel(axFvxImageGray.GetOcx(), axFvxImageBin.GetOcx(), 128);
                    axFvxDisplayEx1.DrawImage(axFvxImageBin.GetOcx());
                }
            }
            else
            {
                //終了
                Application.Exit();
                return;
            }
        }
        //終了時
        private void frmMain_FormClosed(object sender, FormClosedEventArgs e)
        {
            if (m_blnVideoOpened)
                axFvxVideo1.CloseDevice();
        }
        //カメラ取込
        private void btnGrab_Click(object sender, EventArgs e)
        {
            //画像ファイル読み込みによって、画像サイズが初期化時から
            //変更されている可能性があるため、ChangeImageSizeを行います。
            //画像サイズが変更される可能性が無い場合は、初期化時に一度だけ行えば問題有りません。
            
			int BinLevel;
			if (int.TryParse(txtBinLevel.Text, out BinLevel) == false)
			{
				txtBinLevel.Text = "128";
				BinLevel = 128;
			}

            axFvxVideo1.ChangeImageSize(axFvxImageGray.GetOcx());

            try
            {
                //同期取込
                axFvxVideo1.GrabSync(axFvxImageGray.GetOcx());
            }
            catch (Exception)
            {
                int IntErr;
                axFvxVideo1.GetErrReport(out IntErr);
                MessageBox.Show("ビデオの取込に失敗しました｡ ErrCode= " + IntErr.ToString());
            }

            //取り込んだ画像を画面に表示
            axFvxChangeImage1.FixedLevel(axFvxImageGray.GetOcx(), axFvxImageBin.GetOcx(), BinLevel);
            axFvxDisplayEx1.DrawImage(axFvxImageBin.GetOcx());
        }

        //画像ファイル読込
        private void btnLoadImage_Click(object sender, EventArgs e)
        {
            // ダイアログにて画像ファイルを選択します｡
            openFileDialog1.InitialDirectory = m_strDirectory;
            openFileDialog1.Filter = "bitmap file(*.bmp)|*.bmp";
            openFileDialog1.DefaultExt = "*.bmp";
            openFileDialog1.ShowDialog();

			int BinLevel;
			if (int.TryParse(txtBinLevel.Text, out BinLevel) == false)
			{
				txtBinLevel.Text = "128";
				BinLevel = 128;
			}

            if (openFileDialog1.FileName.Length > 0)
            {
                //FileAccessコンポーネントを用いて、
                //ファイルからImageに画像を読み込みます｡
                axFvxFileAccess1.FilePath = openFileDialog1.FileName;

                try
                {
                    axFvxFileAccess1.LoadBitmap(axFvxImageGray.GetOcx());
                }
                catch (Exception)
                {
                    int IntErr = 0;
                    axFvxFileAccess1.GetErrReport(out IntErr);
                    MessageBox.Show("読込みに失敗しました｡ ErrCode= " + IntErr.ToString());
                    return;
                }

                //画像タイプチェック
                if (axFvxImageGray.ImageType != FVXCOMIMAGELib.FVXIMAGE_IMAGE_TYPE.imageGray || axFvxImageGray.ImageWidth != 512 || axFvxImageGray.ImageHeight != 480)
                {
                    MessageBox.Show("512*480 1/8bitグレイの画像ではありません｡", "error", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    axFvxImageGray.BitPerPixel = 8;
                    axFvxImageGray.ImageType = FVXCOMIMAGELib.FVXIMAGE_IMAGE_TYPE.imageGray;
                    axFvxImageGray.ImageWidth = 512;
                    axFvxImageGray.ImageHeight = 480;
                }

                //2値か濃淡か
                if (axFvxImageGray.BitPerPixel == 1)
                {
                    axFvxImageGray.ClearImage();
                    axFvxImageGray.CopyTo(axFvxImageBin.GetOcx());
                    axFvxDisplayEx1.DrawImage(axFvxImageBin.GetOcx());
                }
                else if (axFvxImageGray.BitPerPixel == 8)
                {
                    axFvxImageGray.CopyTo(axFvxImageGray.GetOcx());
                    axFvxChangeImage1.FixedLevel(axFvxImageGray.GetOcx(), axFvxImageBin.GetOcx(), BinLevel);
                    axFvxDisplayEx1.DrawImage(axFvxImageBin.GetOcx());
                }
                else
                    MessageBox.Show("8bit以外のbmpです", "error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }


        private void btnExecute_Click(object sender, EventArgs e)
        {
            axFvxDisplayEx1.ClearOverlay();

            //面積下限・上限設定
            int MinSize,MaxSize;
            if (int.TryParse(txtMinSize.Text, out MinSize) == false || int.TryParse(txtMaxSize.Text, out MaxSize) == false || MinSize < 1)
            {
                MessageBox.Show("最小面積/最大面積は 1 以上の数値を設定して下さい", "パラメータエラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            else if (MinSize >= MaxSize)
            {
                MessageBox.Show("最小面積値＜最大面積値となるよう設定して下さい", "パラメータエラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            axFvxBlob1.MinSize = MinSize;
            axFvxBlob1.MaxSize = MaxSize;

            //対象色設定
            axFvxBlob1.Color = (FVXCOMBLOBLib.COLOR_TYPE)cmbColor.SelectedIndex;

            //配置順設定
            RadioButton radio = new RadioButton();
            foreach (Control control in  groupBoxArrange.Controls)//グループボックス内のコントロール全て見る
            {
                if (control.GetType() == radio.GetType())　//グループ内のラジオボタンだけ
                {
                    radio = (RadioButton)control;
                    if (radio.Checked == true)
                    {
                        axFvxBlob1.Arrange = (FVXCOMBLOBLib.ARRANGE_TYPE)Convert.ToInt32(radio.Tag);
                    }
                }
            }

            //処理範囲接触許可
            if (chkWindTouch.Checked == true)
                axFvxBlob1.WindowTouch = FVXCOMBLOBLib.WINDOW_TOUCH_TYPE.ON;
            else
                axFvxBlob1.WindowTouch = FVXCOMBLOBLib.WINDOW_TOUCH_TYPE.OFF;


            //処理範囲設定
            axFvxBlob1.SetInspectArea(m_ExeRegion);
            axFvxDisplayEx1.PenColor = Color.Lime;
            axFvxDisplayEx1.DrawData(0, m_ExeRegion); 

            int Count = 0;
            double dblTime = 0;
            int IntErr;

            try
            {
                axFvxPerformanceCounter1.StartCount();
                //2値計測実行
                axFvxBlob1.Execute(axFvxImageBin.GetOcx(), out Count);
                axFvxPerformanceCounter1.StopCount(out dblTime);

                txtResult.Text = string.Format("処理時間 : {0:0.00ms}\r\n検出個数 : {1:0}",dblTime,Count);
                axFvxDisplayEx1.PenColor = Color.Blue;
                axFvxDisplayEx1.FontColor = Color.Red;

                if (Count >= 2)
                {
                    FvBlobResult BlobResult = new FvBlobResult();
                    FvBlobResult MaxBlob = new FvBlobResult();
                    FvBlobResult Max2Blob = new FvBlobResult();
                    FvPoint Point = new FvPoint();
                    FvLineSegment LineSeg = new FvLineSegment();
                    double dblDistance = 0;

                    for (int i = 0; i < Count; i++)
                    {
                        //結果の取得
                        axFvxBlob1.GetResult(i, BlobResult);

                        //重心点と外接長方形を描画します
                        axFvxDisplayEx1.DrawData(0, BlobResult);

                        Point.x = BlobResult.center_x;
                        Point.y = BlobResult.center_y;
                        axFvxDisplayEx1.DrawText(0, Point, i.ToString());

                        //1番目と2番目に面積の大きいblobを抽出します。
                        if (BlobResult.area > MaxBlob.area)
                        {
                            CopyBlobResult(Max2Blob, MaxBlob);
                            CopyBlobResult(MaxBlob, BlobResult);

                        }
                        else if (BlobResult.area > Max2Blob.area)
                        {
                            CopyBlobResult(Max2Blob, BlobResult);
                        }
                    }

                    //2点を結ぶ直線を赤で描画します。
                    LineSeg.xs = MaxBlob.center_x;
                    LineSeg.ys = MaxBlob.center_y;
                    LineSeg.xe = Max2Blob.center_x;
                    LineSeg.ye = Max2Blob.center_y;

                    axFvxDisplayEx1.PenColor = Color.Red;
                    axFvxDisplayEx1.DrawData(0, LineSeg);

                    //2点間の距離を計算します。
                    // 点１(x1,y1) 点２(x2,y2)とすると、
                    // 距離 = √((x2-x1)^2 +(y2-y1)^2)
                    
                    dblDistance = System.Math.Sqrt(System.Math.Pow((MaxBlob.center_x - Max2Blob.center_x), 2) +(System.Math.Pow((MaxBlob.center_y - Max2Blob.center_y), 2)));

                    //結果をテキストボックスに表示します
                    txtResult.Text = txtResult.Text + string.Format("\r\n\r\n重心座標1 ( {0:0.0} , {1:0.0} )\r\n重心座標2 ( {2:0.0} , {3:0.0} )\r\n距離 : {4:0.00}", MaxBlob.center_x, MaxBlob.center_y, Max2Blob.center_x, Max2Blob.center_y, dblDistance);
                }
                else    MessageBox.Show("ブローブが2つ以上見つかりませんでした", "エラー", MessageBoxButtons.OK, MessageBoxIcon.Error);             

            }
            //COMエラーを取得
            catch (System.Runtime.InteropServices.COMException)
            {
                string ErrMsg;
                // エラーが出ているか確認	
                // 必ずGetErrReportメソッドを使ってエラー番号を取得してください

                axFvxBlob1.GetErrReport(out IntErr);

                switch (IntErr)
                {
                    case 3001:
                        ErrMsg = "対象画像が指定されていません";
                        break;
                    case 3005:
                        ErrMsg = "パラメータが違います";
                        break;
                    case 3006:
                        ErrMsg = "引数の方が違います。";
                        break;
                    case 3007:
                        ErrMsg = "引数にNULLが指定されています";
                        break;
                    case 3010:
                        ErrMsg = "メモリの確保に失敗しました";
                        break;
                    case 3012:
                        ErrMsg = "ブローブが多すぎます";
                        break;
                    case 3013:
                        ErrMsg = "テーブルの作成に失敗しました";
                        break;
                    case 3014:
                        ErrMsg = "ブローブが１つもありません";
                        break;
                    default:
                        ErrMsg = "不明なエラー";
                        break;

                }
                MessageBox.Show(ErrMsg, "2値計測エラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            //システムエラーを取得
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "システムエラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
        }

        //処理範囲設定
        private void btnExeRegion_Click(object sender, EventArgs e)
        {
            int IntOK = 0;
            double dblTemp = 0.0;

            axFvxROI1.SetRectangle(m_ExeRegion);

            // FvxROIを使って処理範囲選択ダイアログを出します｡
            axFvxROI1.UsePoint = false;
            axFvxROI1.MakeROIDialog(axFvxImageBin.GetOcx(), out IntOK);

            if ((IntOK == 1) && (axFvxROI1.CheckRectangle == 1))
            {
                axFvxROI1.GetRectangle(m_ExeRegion);
                // 始点と終点が逆になっていないかチェックします｡
                // FvxROIは矩形データで、左上・右下の座標位置(xs > xe, ys > ye)が
                // 逆になっていた場合でも、入れ替えずそのまま結果を返しますので、注意してください
                if (m_ExeRegion.xe < m_ExeRegion.xs)
                {
                    dblTemp = m_ExeRegion.xs;
                    m_ExeRegion.xs = m_ExeRegion.xe;
                    m_ExeRegion.xe = dblTemp;
                }
                if (m_ExeRegion.ye < m_ExeRegion.ys)
                {
                    dblTemp = m_ExeRegion.ys;
                    m_ExeRegion.ys = m_ExeRegion.ye;
                    m_ExeRegion.ye = dblTemp;
                }
            }
        }

        private void btnSaveImage_Click(object sender, EventArgs e)
        {
            saveFileDialog1.InitialDirectory = m_strDirectory;
            saveFileDialog1.Filter = "bitmap file(*.bmp)|*.bmp";
            saveFileDialog1.DefaultExt = "*.bmp";
            saveFileDialog1.ShowDialog();

            if (saveFileDialog1.FileName.Length > 0)
            {
                //FileAccessコンポーネントを使って
                //Imageをファイルに保存します｡
                axFvxFileAccess1.FilePath = saveFileDialog1.FileName;
                try
                {
                    axFvxFileAccess1.SaveBitmap(axFvxImageGray.GetOcx());
                }
                catch (Exception)
                {
                    int IntErr = 0;
                    axFvxFileAccess1.GetErrReport(out IntErr);
                    MessageBox.Show("保存に失敗しました。　ErrCode= " + IntErr.ToString());
                    return;
                }
            }
        }

        private void btnClear_Click(object sender, EventArgs e)
        {
            txtResult.Text = "";
            axFvxDisplayEx1.ClearOverlay();
        }

        private void txtBinLevel_TextChanged(object sender, EventArgs e)
        {
            int length = txtBinLevel.Text.Length;

            if (length != 0)
            {
                int BinLevel = int.Parse(txtBinLevel.Text);

                if (BinLevel > 255)
                {
                    BinLevel = 255;
                    txtBinLevel.Text = "255";
                }
                axFvxChangeImage1.FixedLevel(axFvxImageGray.GetOcx(), axFvxImageBin.GetOcx(), BinLevel);
                axFvxDisplayEx1.DrawImage(axFvxImageBin.GetOcx());
            }
        }

        private void txtBinLevel_KeyPress(object sender, KeyPressEventArgs e)
        {
            if ((e.KeyChar < '0' || e.KeyChar > '9') && e.KeyChar != '\b')
                e.Handled = true;

        }

        private void txtMinSize_KeyPress(object sender, KeyPressEventArgs e)
        {
            if ((e.KeyChar < '0' || e.KeyChar > '9') && e.KeyChar != '\b')
                e.Handled = true;

        }
          
        private void txtMaxSize_KeyPress(object sender, KeyPressEventArgs e)
        {
            if ((e.KeyChar < '0' || e.KeyChar > '9') && e.KeyChar != '\b')
                e.Handled = true;
        }
        private void CopyBlobResult(FvBlobResult pDst, FvBlobResult pSrc)
        {
            pDst.area = pSrc.area;
		    pDst.bounding_bottom_x = pSrc.bounding_bottom_x;
		    pDst.bounding_bottom_y = pSrc.bounding_bottom_y;
		    pDst.bounding_left_x = pSrc.bounding_left_x;
		    pDst.bounding_left_y = pSrc.bounding_left_y;
		    pDst.bounding_right_x = pSrc.bounding_right_x;
		    pDst.bounding_right_y = pSrc.bounding_right_y;
		    pDst.bounding_top_x = pSrc.bounding_top_x;
		    pDst.bounding_top_y = pSrc.bounding_top_y;
		    pDst.center_x = pSrc.center_x;
		    pDst.center_y = pSrc.center_y;
		    pDst.ellipse_angle = pSrc.ellipse_angle;
		    pDst.ellipse_long = pSrc.ellipse_long;
		    pDst.ellipse_short = pSrc.ellipse_short;
		    pDst.hole_num = pSrc.hole_num;
		    pDst.perim = pSrc.perim;
		    pDst.total_area = pSrc.total_area;
		    pDst.x_max = pSrc.x_max;
		    pDst.x_min = pSrc.x_min;
		    pDst.y_max = pSrc.y_max;
		    pDst.y_min = pSrc.y_min;
        }
 
    }
}